<?php

namespace PrestaShop\Module\LCVPrestaConnector\Forms;

use LCVPrestaConnector;
use PrestaShop\Module\LCVPrestaConnector\SyncConfiguration;
use PrestaShopBundle\Form\Admin\Type\CustomContentType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\AbstractType;
use PrestaShopBundle\Form\Admin\Type\SwitchType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;

class catalogType extends AbstractType {

    /**
     * lien vers le module
     */
    protected LCVPrestaConnector $module;

    public function __construct() {
        $this->module = \Module::getInstanceByName(strtolower('LCVPrestaConnector'));
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $backofficeName = $this->module->getBridge()->getId();
        $cfg = $this->module->getCfg();

        $builder->add('title_stock', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => false,
                'title' => 'Options de stock',
                'help' => '',
            ],
        ]);

        /**
         * Stock de sécurité
         */
        $builder
            ->add('secured_stock', TextType::class, [
                'label' => 'Stock de sécurité',                
                'required' => false,
                'help' => 'Cette valeur est déduite du stock de chaque taille de produit lors de l\'importation du stock.', // should be set to enable help block
                'data' => $this->module->getCfg()->get(SyncConfiguration::CFG_SECURED_STOCK),
            ]);

        /**
         * Cacher/montrer les produits en stock
         */
        $builder
            ->add('hide_stock_method', ChoiceType::class, [
                'label' => 'Visibilité des produits en stock',                
                //'choice_attr' => $this->productVisibilityChoiceProvider->getChoicesAttributes(),
                'choices' => [
                    'Ne pas changer la visibilité du produit' => '',
                    'Produit visible' => 'both',
                    'Cacher le produit' => 'none',
                    'Produit uniquement visible dans le parcours catalogue' => 'catalog',
                    'Produit uniquement visible lors d\'une recherche' => 'search',
                ],

                'expanded' => false,
                'required' => false,
                // placeholder false is important to avoid empty option in radio select despite required being false
                'placeholder' => false,
                'column_breaker' => true,
                'help' => '', // should be set to enable help block
                'data' => $this->module->getCfg()->get(SyncConfiguration::CFG_NO_STOCK_PRODUCT_VISIBILITY ),
            ]);        

        /**
         * CFG_PR_ATTRIBUTES_MAPPING
         */
        $productMapping = $cfg->get(SyncConfiguration::CFG_PR_ATTRIBUTES_MAPPING);
        $productAttrs = $cfg->get(SyncConfiguration::CFG_DYN_ATTR);
        $productAttrsExtended = array_merge($productAttrs, $cfg->get(SyncConfiguration::CFG_DYN_CAT));

        /** Mapping des produits */
        $builder->add('title_product_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Attributs discriminants',
                'help' => '<p>
                                Sélectionnez les attributs '.$backofficeName.' que vous souhaitez utiliser comme discriminants de produit Prestashop (cf. documentation).<br>
                                <em>Par exemple, si vous désirez créer un produit Prestashop par couleur, activez "couleur".</em>
                                <br>
                                <em><strong>Notes : les attributs non mappés sont obligatoirement discriminants.</strong></em>
                            </p>',
            ],
        ]);

        // Création dynamique des champs en fonction des options de configuration
        foreach ($productAttrs as $k => $name)
        {
            $safe_key = 'PRODUCT_ATTR_PRODUCT_'.trim(base64_encode($k), '=');
            $builder->add($safe_key, SwitchType::class, [
                'label' => $name,
                'required' => false,
                'data' => $productMapping[SyncConfiguration::PRODUCT_ATTR_PRODUCT][$k] ?? false, // valeur par défaut selon l'option configurée
            ]);
        }

        /** Mapping des attributs */
        $builder->add('title_attribute_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Mappage des attributs de produit',
                'help' => '<p>
                                Pour chaque type d\'attributs de votre back-office '.$backofficeName.' sélectionnez le <strong>groupe d\'attributs Prestashop</strong> dans lequel vous souhaitez reporter la valeur.
                                <br>
                                Si vous indiquez "---", l\'attribut ne sera pas synchronisé dans un groupe d\'attributs.
                            </p>',
            ],
        ]);

        // Création dynamique des champs en fonction des options de configuration
        $attributeGroups = ['---' => 0];

        // Pour empêcher les erreurs, on retire tous les groupes d'attributs qui ont été affectés à des tailles
        $sizeGrids = $cfg->get(SyncConfiguration::CFG_MAP_SIZEGRIDS) ?? [ '*' => -1 ];

        foreach (\AttributeGroup::getAttributesGroups(\Context::getContext()->language->id) as $ag)
            if (!in_array($ag['id_attribute_group'], $sizeGrids))
                $attributeGroups[$ag['name']] = $ag['id_attribute_group'];

        foreach ($productAttrsExtended as $k => $name)
        {
            $safe_key = 'PRODUCT_ATTR_VARIANT_'.trim(base64_encode($k), '=');
            $builder->add($safe_key, ChoiceType::class, [
                'label' => $name,
                'required' => false,
                'expanded' => false,
                'multiple' => false,
                'empty_data' => 0,
                'choices' => $attributeGroups,
                'data' => $productMapping[SyncConfiguration::PRODUCT_ATTR_VARIANT][$k] ?? 0, // valeur par défaut selon l'option configurée
            ]);
        }

        /** Mapping des caractéristiques */
        $builder->add('title_features_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Mappage des caractéristiques de produit',
                'help' => '<p>
                                Pour chaque type d\'attributs de votre back-office '.$backofficeName.' sélectionnez la <strong>caractéristique Prestashop</strong> dans laquelle vous souhaitez reporter la valeur.
                                <br>
                                Si vous n\'indiquez "---", l\'attribut ne sera pas synchronisé dans une caractéristique.
                            </p>',
            ],
        ]);

        // Création dynamique des champs en fonction des options de configuration
        $features = ['---' => 0];
        foreach (\Feature::getFeatures(\Context::getContext()->language->id) as $feat)
            $features[$feat['name']] = $feat['id_feature'];

        foreach ($productAttrsExtended as $k => $name)
        {
            $safe_key = 'PRODUCT_ATTR_FEATURE_'.trim(base64_encode($k), '=');
            $builder->add($safe_key, ChoiceType::class, [
                'label' => $name,
                'required' => false,
                'expanded' => false,
                'multiple' => false,
                'empty_data' => 0,
                'choices' => $features,
                'data' => $productMapping[SyncConfiguration::PRODUCT_ATTR_FEATURE][$k] ?? 0, // valeur par défaut selon l'option configurée
            ]);
        }

        /** Mapping des couleurs */
        $builder->add('title_colors_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Mappage des couleurs',
                'help' => '<p>
                                Lorsqu\'elle est activée, cette option vous permet de substituer la couleur de votre back-office '.$backofficeName.' par une couleur Prestashop (les correspondances sont définissables dans les mappages catalogue).<br>
                                Cette option est utile si vous avez renseigné des nuances sur votre backoffice (jaune moutarde, bordeaux, bleu marine, etc.) et que vous souhaitez les remplacer par des couleurs standardisées (jaune, rouge, bleu, etc.) afin
                                de limiter le nombre de filtres couleurs. <br>
                                Les couleurs du style <code>bleu/blanc</code> sont systématiquement remplacées par la couleur "multicolore" dans Prestashop.
                            </p><p>
                                Une configuration classique consiste à importer la couleur dans un attribut couleur et de sélectionner cet attribut ci-dessous.
                                Pour conserver la nuance, l\'astuce consiste à définir une deuxième importation de la couleur, mais dans une caractéristique "couleur". 
                                Si vous n\'indiquez "Ne pas utiliser le mappage couleur", l\'option sera désactivée.
                            </p>',
            ],
        ]);

        unset($attributeGroups['---']);
        $attributeGroups['Ne pas utiliser le mappage couleur'] = 0;
        $builder->add('COLOR_MAP_ATTRIBUTE', ChoiceType::class, [
            'label' => 'Groupe d\'attributs à convertir',
            'required' => false,
            'expanded' => false,
            'multiple' => false,
            'empty_data' => 0,
            'choices' => $attributeGroups,
            'data' => $cfg->get(SyncConfiguration::CFG_MAP_COLOR_ATTR) ?? 0, // valeur par défaut selon l'option configurée
        ]);

        /**
         * Options de synchronisation des fiches produits
         */
        $builder->add('title_sync', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Données produits synchronisées',
                'help' => '<p>Sélectionnez ici les informations que vous désirez synchroniser depuis votre back-office Polaris vers votre boutique en ligne :</p>
                            <ul>
                                <li>les informations sélectionnées sont systématiquement remplacées par celles de votre back-office lors de la synchronisation du produit ;</li>
                                <li>celles qui ne le sont pas sont uniquement positionnées lors de la création du produit.</li>
                            </ul>
                            ',
            ],
        ]);
        
        $cfgOptions = $cfg->get(SyncConfiguration::CFG_DATA_SYNC);        
        $optionsDesc = $cfg->describeSyncConfiguration();
        $mode = $cfg->get(SyncConfiguration::CFG_WORKMODE);

        // Création dynamique des champs en fonction des options de configuration
        foreach ($optionsDesc as $field => $p)
        {
            $isActive = isset($cfgOptions[$field]) ? $cfgOptions[$field] : false;
            $builder->add($field, SwitchType::class, [
                'label' => $p['label'],
                //'help' => $p['description'],  // Utilisation de la description pour l'aide                
                'required' => false,
                'disabled' => $mode === 0,
                'data' => $isActive, // valeur par défaut selon l'option configurée
            ]);
        }
        
        $this->module->getBridge()->hookFormCatalog($builder, $options);

        // Bouton de sauvegarde
        $builder->add('Enregistrer', SubmitType::class, [
            'attr' => ['class' => 'btn-form-valid btn-primary']
        ]);
    }
}